﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.IO;
using System.Linq;
using System.Text;

namespace SetCBLUserImage
{
    public class UserImage
    {
        private IntelHexFile _file;

        public BindingList<UserImageRecord> Files { get; private set; }

        public byte[] Data
        {
            get
            {
                return _file.Data;
            }
        }

        public UserImage(IntelHexFile file, bool ignoreChecksums)
        {
            _file = file;
            Files = new BindingList<UserImageRecord>();
            Files.RaiseListChangedEvents = false;
            Files.ListChanged += Files_ListChanged;

            //Extract files from data
            int offset = 0;
            while (offset < _file.Data.Length)
            {
                var marker = (ushort)((_file.Data[offset + 1] << 8) | _file.Data[offset]);
                if (marker == 0x37AE)
                {
                    var unknown1 = (ushort)((_file.Data[offset + 11] << 8) | _file.Data[offset + 10]);
                    var type = (UserImageRecord.RecordType)((_file.Data[offset + 3] << 8) | _file.Data[offset + 2]);
                    var size = (ushort)((_file.Data[offset + 5] << 8) | _file.Data[offset + 4]);
                    var checksum = (ulong)((_file.Data[offset + 6] | (_file.Data[offset + 7] << 8) |
                        (_file.Data[offset + 8] << 16) | (_file.Data[offset + 9] << 24)) & 0xFFFFFFFF);
                    size = (ushort)((size - 6) * 2);
                    var data = new byte[size];
                    Array.Copy(_file.Data, offset + 12, data, 0, data.Length);

                    if (ignoreChecksums)
                    {
                        Files.Add(UserImageRecord.Parse(marker, type, unknown1, data, null));
                    }
                    else
                    {
                        Files.Add(UserImageRecord.Parse(marker, type, unknown1, data, checksum));
                    }

                    offset += size + 12;
                }
                else
                {
                    throw new FileLoadException("Invalid user record found: " + marker.ToString("X04"));
                }
            }

            //Enable events
            Files.RaiseListChangedEvents = true;
        }

        public byte[] GetSaveData()
        {
            return _file.GetSaveData();
        }

        private void Files_ListChanged(object sender, ListChangedEventArgs e)
        {
            _RefreshData();
        }

        private void _RefreshData()
        {
            var stream = new MemoryStream();

            //Build data based on files specified
            foreach (var file in Files)
            {
                var data = file.GetSaveData();
                stream.Write(data, 0, data.Length);
            }

            _file.Data = stream.ToArray();
        }
    }
}
